package org.warp.commonutils.concurrency.executor;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.locks.StampedLock;
import org.warp.commonutils.concurrency.atomicity.Atomic;

@Atomic
public class ScheduledTaskLifecycle {

	private final StampedLock lock;
	private final ConcurrentHashMap<ScheduledFuture<?>, Object> tasks = new ConcurrentHashMap<>();

	public ScheduledTaskLifecycle() {
		this.lock = new StampedLock();
	}

	public void registerScheduledTask(ScheduledFuture<?> task) {
		this.tasks.put(task, new Object());
	}

	public void startScheduledTask() {
		this.lock.readLock();
	}

	public void endScheduledTask() {
		this.lock.tryUnlockRead();
	}

	public void cancelAndWait() {
		tasks.forEach((task, obj) -> {
			task.cancel(false);
		});

		lock.unlockWrite(lock.writeLock());
	}
}
